/* Name: alignalg.m                                                          **
**                                                                           **
**                                                                           **
** Funktion: Implementation der Kategorie AlignmentAlgorithms, die der       **
**           Klasse NLet Funktionen zum Alignieren von Sequenzen             **
**           bereitstellt                                                    **
**                                                                           **
** Aufruf: -- (Einbindung in ObjC-Dateien ueber #import bzw. #include)       **
**                                                                           **
** Probleme:                                                                 **
**                                                                           **
** Erstellt: 24.08.1995                                                      **
** Zuletzt bearbeitet: 23.10.1996                                            */

/* ------------------------------------------------------------------------- */


/* Include-Dateien                                                           */

#include "alignalg.h"

#include "matrint.h"
#include "error.h"
#include "constant.h"

#include <ctype.h>
#include <string.h>
#include <stdlib.h>

/* Konstanten- und Typdefinitionen                                           */

#define NWNOSTEP -128

/* Klasse: AlignmentAlgorithms                                               */

@implementation NLet (AlignmentAlgorithms)

+ (Alignment*)needlemanWunschOf:firstNLet0 from:(long)firstFrom0 to:(long)firstTo0 and:secondNLet0 from:(long)secondFrom0 to:(long)secondTo0 withMatrix:matrix0 firstGapPenalty:(int)firstGap0 restGapPenalty:(int)restGap0
{
  Alignment *newAlignment;
  long **qualityMatrix;
  char **pathMatrix;
  char **alignArray;
  int* qualityRow;
  long from1, to1, from2, to2;
  long len1, len2, len, maxAlignmentLength;
  long breakoff, start, currentI, currentJ;
  long bestQuality, currentQuality;
  long bestInRow, *bestInColumn;
  long i, j, k;
  if(![firstNLet0 isKindOf:[NLet class]] || ![secondNLet0 isKindOf:[NLet class]] || ![matrix0 isKindOf:[MatrixInteger class]])
    {
      [ErrorHandler error:ERRMETHODWRONGOBJ];
      return nil;
    }
  if([matrix0 getRows]<NUMLETTER || [matrix0 getColumns]<NUMLETTER)
    {
      [ErrorHandler error:ERRMETHODWRONGOBJ];
      return nil;
    }
  if(firstFrom0<0)
    from1=0;
  else 
    from1=firstFrom0;
  if(secondFrom0<0)
    from2=0;
  else 
    from2=secondFrom0;
  if(firstTo0>=[firstNLet0 getLength])
    to1=[firstNLet0 getLength]-1;
  else 
    to1=firstTo0;
  if(secondTo0>=[secondNLet0 getLength])
    to2=[secondNLet0 getLength]-1;
  else 
    to2=secondTo0;
  if(from1>to1 || from2>to2)
    {
      [ErrorHandler error:ERRMETHODFAIL];
      return nil;
    }
  len1=to1-from1+1;
  len2=to2-from2+1;
  maxAlignmentLength=len1+len2;
  qualityMatrix=(long**)calloc(len1,sizeof(long*));
  for(i=0;i<len1;i++)
    qualityMatrix[i]=(long*)calloc(len2,sizeof(long));
  pathMatrix=(char**)calloc(len1,sizeof(char*));
  for(i=0;i<len1;i++)
    pathMatrix[i]=(char*)calloc(len2,sizeof(char));
  alignArray=(char**)malloc(2*sizeof(char*));
  alignArray[0]=(char*)calloc(maxAlignmentLength+1,sizeof(char));
  alignArray[1]=(char*)calloc(maxAlignmentLength+1,sizeof(char));
  qualityRow=(int*)calloc(maxAlignmentLength,sizeof(int));
  bestInColumn=(long*)calloc(len2,sizeof(long));
  if(!qualityMatrix[len1-1] || !pathMatrix[len1-1] || !qualityRow || !bestInColumn)
    {
      [ErrorHandler error:ERRMEMNOTENOUGH];
      return nil;
    }
  /* Beginn des eigentlichen Algorithmus */
  /* 1. Schritt: Initialisierung der Matrix */
  for(i=0;i<len1;i++)
    for(j=0;j<len2;j++)
      {
	qualityMatrix[i][j]=(long)[matrix0 getElementAtRow:toupper([firstNLet0 getSequenceAt:from1+i])-ASCIIA column:toupper([secondNLet0 getSequenceAt:from2+j])-ASCIIA];
	pathMatrix[i][j]=NWNOSTEP;
      }
  bestInRow=len2-1;
  for(j=0;j<len2;j++)
    bestInColumn[j]=len1-1;
  /* 2. Schritt: Durchschreiten der Matrix; Aktualisieren der Qualitaetswerte */
  for(i=len1-2;i>=0;i--)
    {
      bestInRow=len2-1;
      for(j=len2-2;j>=0;j--)
	{
	  bestQuality=qualityMatrix[i+1][j+1];
	  pathMatrix[i][j]=0;
	  /* for(k=i+2;k<len1 && (k-i-1)<=MAXALIGNGAPLENGTH;k++)
	    {
	      currentQuality=qualityMatrix[k][j+1]-(long)firstGap0-(k-i-2)*(long)restGap0;
	      if(currentQuality>bestQuality)
		{
		  bestQuality=currentQuality;
		  pathMatrix[i][j]=k-i-1;
		}
	    }
	  for(k=j+2;k<len2 && (k-j-1)<=MAXALIGNGAPLENGTH;k++)
	    {
	      currentQuality=qualityMatrix[i+1][k]-(long)firstGap0-(k-j-2)*(long)restGap0;
	      if(currentQuality>bestQuality)
		{
		  bestQuality=currentQuality;
		  pathMatrix[i][j]=-k+j+1;
		}
	    } */
	  currentQuality=qualityMatrix[bestInColumn[j+1]][j+1]-(long)firstGap0-(bestInColumn[j+1]-i-2)*(long)restGap0;
	  if(currentQuality>bestQuality)
	    {
	      bestQuality=currentQuality;
	      pathMatrix[i][j]=bestInColumn[j+1]-i-1;
	    }
	  else
	    if(qualityMatrix[i+1][j+1]+(bestInColumn[j+1]-i-1)*(long)restGap0>=qualityMatrix[bestInColumn[j+1]][j+1])
	      bestInColumn[j+1]=i+1;
	  currentQuality=qualityMatrix[i+1][bestInRow]-(long)firstGap0-(bestInRow-j-2)*(long)restGap0;
	  if(currentQuality>bestQuality)
	    {
	      bestQuality=currentQuality;
	      pathMatrix[i][j]=-bestInRow+j+1;
	    }
	  else
	    if(qualityMatrix[i+1][j+1]+(bestInRow-j-1)*(long)restGap0>=qualityMatrix[i+1][bestInRow])
	      bestInRow=j+1;
	  qualityMatrix[i][j]=qualityMatrix[i][j]+bestQuality;
	}
    }
  /* 3. Schritt: Finden des Ausgangspunktes fuer den besten Weg */
  bestQuality=qualityMatrix[0][0];
  start=0;
  breakoff=len1+len2; /* Abbruchkriterium ? */
  for(i=1;i<=breakoff;i++)
    {
      if(i%2)
	if(i/2+1<len1 && qualityMatrix[i/2+1][0]>bestQuality)
	  {
	    bestQuality=qualityMatrix[i/2+1][0];
	    start=i/2+1;
	  }
      else
	if(i/2<len2 && qualityMatrix[0][i/2]>bestQuality)
	  {
	    bestQuality=qualityMatrix[0][i/2];
	    start=-i/2;
	  }
    }
  /* 4. Schritt: Zusammenstellen des besten Weges */
  /* 4.1.: Weg bis zum Anfangspunkt des gemeinsamen Teils */
  if(start>0)
    {
      strcpy(alignArray[0],[firstNLet0 getSequenceFrom:from1 to:from1+start-1]);
      for(i=0;i<start;i++)
	{
	  alignArray[1][i]=ALIGNGAPCHAR;
	  qualityRow[i]=0;
	}
      len=start;
      currentI=start;
      currentJ=0;
    }
  else
    if(start<0)
      {
	strcpy(alignArray[1],[secondNLet0 getSequenceFrom:from2 to:from2-start-1]);
	for(j=0;j<-start;j++)
	  {
	    alignArray[0][j]=ALIGNGAPCHAR;
	    qualityRow[j]=0;
	  }
	len=-start;
	currentI=0;
	currentJ=-start;
      }
    else
      {
	len=0;
	currentI=currentJ=0;
      }
  /* 4.2.: Weg im gemeinsamen Teil */
  while(currentI<len1 && currentJ<len2)
    {
      alignArray[0][len]=[firstNLet0 getSequenceAt:from1+currentI];
      alignArray[1][len]=[secondNLet0 getSequenceAt:from2+currentJ];
      qualityRow[len]=[matrix0 getElementAtRow:toupper(alignArray[0][len])-ASCIIA column:toupper(alignArray[1][len])-ASCIIA];
      len++;
      if(pathMatrix[currentI][currentJ]==0)
	{
	  currentI++;
	  currentJ++;
	}
      else
	if(pathMatrix[currentI][currentJ]==NWNOSTEP)
	  {
	    if(currentI==len1-1)
	      currentI++;
	    if(currentJ==len2-1)
	      currentJ++;
	  }
	else
	  if(pathMatrix[currentI][currentJ]>0)
	    {
	      strcat(alignArray[0],[firstNLet0 getSequenceFrom:from1+currentI+1 to:from1+currentI+pathMatrix[currentI][currentJ]]);
	      for(i=0;i<pathMatrix[currentI][currentJ];i++)
		{
		  alignArray[1][len+i]=ALIGNGAPCHAR;
		  if(i==0)
		    qualityRow[len+i]=-firstGap0;
		  else
		    qualityRow[len+i]=-restGap0;
		}
	      len+=pathMatrix[currentI][currentJ];
	      currentI+=pathMatrix[currentI][currentJ]+1;
	      currentJ++;
	    }
	  else
	    {
	      strcat(alignArray[1],[secondNLet0 getSequenceFrom:from2+currentJ+1 to:from2+currentJ-pathMatrix[currentI][currentJ]]);
	      for(j=0;j<-pathMatrix[currentI][currentJ];j++)
		{
		  alignArray[0][len+j]=ALIGNGAPCHAR;
		  if(j==0)
		    qualityRow[len+j]=-firstGap0;
		  else
		    qualityRow[len+j]=-restGap0;
		}
	      len+=-pathMatrix[currentI][currentJ];
	      currentJ+=-pathMatrix[currentI][currentJ]+1;
	      currentI++;
	    }
    }
  /* 4.3.: Weg vom Endpunkt des gemeinsamen Teils */
  if(currentI<len1)
    {
      strcat(alignArray[0],[firstNLet0 getSequenceFrom:from1+currentI+1 to:from1+len1-1]);
      for(i=0;i<len1-currentI-1;i++)
	{
	  alignArray[1][len+i]=ALIGNGAPCHAR;
	  qualityRow[len+i]=0;
	}
      len+=len1-currentI-1;
    }
  if(currentJ<len2)
    {
      strcat(alignArray[1],[secondNLet0 getSequenceFrom:from2+currentJ+1 to:from2+len2-1]);
      for(j=0;j<len2-currentJ-1;j++)
	{
	  alignArray[0][len+j]=ALIGNGAPCHAR;
	  qualityRow[len+j]=0;
	}
      len+=len2-currentJ-1;
    }
  /* Ende des eigentlichen Algorithmus */
  newAlignment=[Alignment new:alignArray number:2 length:len];
  newAlignment=[newAlignment setQualityRow:qualityRow];
  for(i=0;i<len1;i++)
    {
      free(qualityMatrix[i]);
      free(pathMatrix[i]);
    }
  free(qualityMatrix);
  free(pathMatrix);
  free(alignArray[0]);
  free(alignArray[1]);
  free(alignArray);
  free(qualityRow);
  free(bestInColumn);
  return newAlignment;
}

+ (Alignment*)neighbourhoodNeedlemanWunschOf:firstNLet0 from:(long)firstFrom0 to:(long)firstTo0 and:secondNLet0 from:(long)secondFrom0 to:(long)secondTo0 withMatrix:matrix0 firstGapPenalty:(int)firstGap0 restGapPenalty:(int)restGap0
{
  Alignment *newAlignment;
  long **qualityMatrix;
  int **neighbourhood;
  char **pathMatrix;
  char **alignArray;
  int* qualityRow;
  long from1, to1, from2, to2;
  long len1, len2, len, maxAlignmentLength;
  long breakoff, start, currentI, currentJ;
  double bestQuality, currentQuality, currentNeighbourhood;
  long bestInRow, *bestInColumn;
  long i, j, k;
  if(![firstNLet0 isKindOf:[NLet class]] || ![secondNLet0 isKindOf:[NLet class]] || ![matrix0 isKindOf:[MatrixInteger class]])
    {
      [ErrorHandler error:ERRMETHODWRONGOBJ];
      return nil;
    }
  if([matrix0 getRows]<NUMLETTER || [matrix0 getColumns]<NUMLETTER)
    {
      [ErrorHandler error:ERRMETHODWRONGOBJ];
      return nil;
    }
  if(firstFrom0<0)
    from1=0;
  else 
    from1=firstFrom0;
  if(secondFrom0<0)
    from2=0;
  else 
    from2=secondFrom0;
  if(firstTo0>=[firstNLet0 getLength])
    to1=[firstNLet0 getLength]-1;
  else 
    to1=firstTo0;
  if(secondTo0>=[secondNLet0 getLength])
    to2=[secondNLet0 getLength]-1;
  else 
    to2=secondTo0;
  if(from1>to1 || from2>to2)
    {
      [ErrorHandler error:ERRMETHODFAIL];
      return nil;
    }
  len1=to1-from1+1;
  len2=to2-from2+1;
  maxAlignmentLength=len1+len2;
  qualityMatrix=(long**)calloc(len1,sizeof(long*));
  for(i=0;i<len1;i++)
    qualityMatrix[i]=(long*)calloc(len2,sizeof(long));
  neighbourhood=(int**)calloc(2,sizeof(int*));
  for(i=0;i<2;i++)
    neighbourhood[i]=(int*)calloc(2*ALIGNALGNEIGHBOURHOOD,sizeof(int));
  pathMatrix=(char**)calloc(len1,sizeof(char*));
  for(i=0;i<len1;i++)
    pathMatrix[i]=(char*)calloc(len2,sizeof(char));
  alignArray=(char**)malloc(2*sizeof(char*));
  alignArray[0]=(char*)calloc(maxAlignmentLength+1,sizeof(char));
  alignArray[1]=(char*)calloc(maxAlignmentLength+1,sizeof(char));
  qualityRow=(int*)calloc(maxAlignmentLength,sizeof(int));
  bestInColumn=(long*)calloc(len2,sizeof(long));
  if(!qualityMatrix[len1-1] || !neighbourhood[1] || !pathMatrix[len1-1] || !qualityRow || !bestInColumn)
    {
      [ErrorHandler error:ERRMEMNOTENOUGH];
      return nil;
    }
  /* Beginn des eigentlichen Algorithmus */
  /* 1. Schritt: Initialisierung der Matrix */
  /* Hier erfolgt der Einbau der Nachbarschaft */
  for(i=0;i<len1;i++)
    for(j=0;j<len2;j++)
      {
	qualityMatrix[i][j]=(long)[matrix0 getElementAtRow:toupper([firstNLet0 getSequenceAt:from1+i])-ASCIIA column:toupper([secondNLet0 getSequenceAt:from2+j])-ASCIIA];
	/* Berechnung der Nachbarschaft */
	for(k=0;k<ALIGNALGNEIGHBOURHOOD;k++)
	  {
	    if(j-ALIGNALGNEIGHBOURHOOD+k>=0)
	      neighbourhood[0][k]=(int)[matrix0 getElementAtRow:toupper([firstNLet0 getSequenceAt:from1+i])-ASCIIA column:toupper([secondNLet0 getSequenceAt:from2+j-ALIGNALGNEIGHBOURHOOD+k])-ASCIIA];
	    else 
	      neighbourhood[0][k]=0;
	    if(j+k+1<len2)
	      neighbourhood[0][ALIGNALGNEIGHBOURHOOD+k]=(int)[matrix0 getElementAtRow:toupper([firstNLet0 getSequenceAt:from1+i])-ASCIIA column:toupper([secondNLet0 getSequenceAt:from2+j+k+1])-ASCIIA];
	    else
	      neighbourhood[0][ALIGNALGNEIGHBOURHOOD+k]=0;
	    if(i-ALIGNALGNEIGHBOURHOOD+k>=0)
	      neighbourhood[1][k]=(int)[matrix0 getElementAtRow:toupper([firstNLet0 getSequenceAt:from1+i-ALIGNALGNEIGHBOURHOOD+k])-ASCIIA column:toupper([secondNLet0 getSequenceAt:from2+j])-ASCIIA];
	    else
	      neighbourhood[1][k]=0;
	    if(i+k+1<len1)
	      neighbourhood[0][ALIGNALGNEIGHBOURHOOD+k]=(int)[matrix0 getElementAtRow:toupper([firstNLet0 getSequenceAt:from1+i+k+1])-ASCIIA column:toupper([secondNLet0 getSequenceAt:from2+j])-ASCIIA];
	    else
	      neighbourhood[1][ALIGNALGNEIGHBOURHOOD+k]=0;
	  }
	currentNeighbourhood=0.0;
	for(k=0;k<ALIGNALGNEIGHBOURHOOD;k++)
	  {
	    currentNeighbourhood+=(double)1/(k+2)*(double)neighbourhood[0][ALIGNALGNEIGHBOURHOOD-k-1];
	    currentNeighbourhood+=(double)1/(k+2)*(double)neighbourhood[0][ALIGNALGNEIGHBOURHOOD+k];
	    currentNeighbourhood+=(double)1/(k+2)*(double)neighbourhood[1][ALIGNALGNEIGHBOURHOOD-k-1];
	    currentNeighbourhood+=(double)1/(k+2)*(double)neighbourhood[1][ALIGNALGNEIGHBOURHOOD+k];
	  }
	qualityMatrix[i][j]+=(long)currentNeighbourhood;
	pathMatrix[i][j]=NWNOSTEP;
      }
  bestInRow=len2-1;
  for(j=0;j<len2;j++)
    bestInColumn[j]=len1-1;
  /* 2. Schritt: Durchschreiten der Matrix; Aktualisieren der Qualitaetswerte */
  for(i=len1-2;i>=0;i--)
    {
      bestInRow=len2-1;
      for(j=len2-2;j>=0;j--)
	{
	  bestQuality=qualityMatrix[i+1][j+1];
	  pathMatrix[i][j]=0;
	  currentQuality=qualityMatrix[bestInColumn[j+1]][j+1]-(long)firstGap0-(bestInColumn[j+1]-i-2)*(long)restGap0;
	  if(currentQuality>bestQuality)
	    {
	      bestQuality=currentQuality;
	      pathMatrix[i][j]=bestInColumn[j+1]-i-1;
	    }
	  else
	    if(qualityMatrix[i+1][j+1]+(bestInColumn[j+1]-i-1)*(long)restGap0>=qualityMatrix[bestInColumn[j+1]][j+1])
	      bestInColumn[j+1]=i+1;
	  currentQuality=qualityMatrix[i+1][bestInRow]-(long)firstGap0-(bestInRow-j-2)*(long)restGap0;
	  if(currentQuality>bestQuality)
	    {
	      bestQuality=currentQuality;
	      pathMatrix[i][j]=-bestInRow+j+1;
	    }
	  else
	    if(qualityMatrix[i+1][j+1]+(bestInRow-j-1)*(long)restGap0>=qualityMatrix[i+1][bestInRow])
	      bestInRow=j+1;
	  qualityMatrix[i][j]=qualityMatrix[i][j]+bestQuality;
	}
    }
  /* 3. Schritt: Finden des Ausgangspunktes fuer den besten Weg */
  bestQuality=qualityMatrix[0][0];
  start=0;
  breakoff=len1+len2; /* Abbruchkriterium ? */
  for(i=1;i<=breakoff;i++)
    {
      if(i%2)
	if(i/2+1<len1 && qualityMatrix[i/2+1][0]>bestQuality)
	  {
	    bestQuality=qualityMatrix[i/2+1][0];
	    start=i/2+1;
	  }
      else
	if(i/2<len2 && qualityMatrix[0][i/2]>bestQuality)
	  {
	    bestQuality=qualityMatrix[0][i/2];
	    start=-i/2;
	  }
    }
  /* 4. Schritt: Zusammenstellen des besten Weges */
  /* 4.1.: Weg bis zum Anfangspunkt des gemeinsamen Teils */
  if(start>0)
    {
      strcpy(alignArray[0],[firstNLet0 getSequenceFrom:from1 to:from1+start-1]);
      for(i=0;i<start;i++)
	{
	  alignArray[1][i]=ALIGNGAPCHAR;
	  qualityRow[i]=0;
	}
      len=start;
      currentI=start;
      currentJ=0;
    }
  else
    if(start<0)
      {
	strcpy(alignArray[1],[secondNLet0 getSequenceFrom:from2 to:from2-start-1]);
	for(j=0;j<-start;j++)
	  {
	    alignArray[0][j]=ALIGNGAPCHAR;
	    qualityRow[j]=0;
	  }
	len=-start;
	currentI=0;
	currentJ=-start;
      }
    else
      {
	len=0;
	currentI=currentJ=0;
      }
  /* 4.2.: Weg im gemeinsamen Teil */
  while(currentI<len1 && currentJ<len2)
    {
      alignArray[0][len]=[firstNLet0 getSequenceAt:from1+currentI];
      alignArray[1][len]=[secondNLet0 getSequenceAt:from2+currentJ];
      /* Guete mit Nachbarschaft berechnen */
      qualityRow[len]=[matrix0 getElementAtRow:toupper(alignArray[0][len])-ASCIIA column:toupper(alignArray[1][len])-ASCIIA];
      for(k=0;k<ALIGNALGNEIGHBOURHOOD;k++)
	{
	  if(currentJ-ALIGNALGNEIGHBOURHOOD+k>=0)
	    neighbourhood[0][k]=(int)[matrix0 getElementAtRow:toupper(alignArray[0][len])-ASCIIA column:toupper([secondNLet0 getSequenceAt:from2+currentJ-ALIGNALGNEIGHBOURHOOD+k])-ASCIIA];
	  else 
	    neighbourhood[0][k]=0;
	  if(currentJ+k+1<len2)
	    neighbourhood[0][ALIGNALGNEIGHBOURHOOD+k]=(int)[matrix0 getElementAtRow:toupper(alignArray[0][len])-ASCIIA column:toupper([secondNLet0 getSequenceAt:from2+currentJ+k+1])-ASCIIA];
	  else
	    neighbourhood[0][ALIGNALGNEIGHBOURHOOD+k]=0;
	  if(currentI-ALIGNALGNEIGHBOURHOOD+k>=0)
	    neighbourhood[1][k]=(int)[matrix0 getElementAtRow:toupper([firstNLet0 getSequenceAt:from1+currentI-ALIGNALGNEIGHBOURHOOD+k])-ASCIIA column:toupper(alignArray[1][len])-ASCIIA];
	  else
	    neighbourhood[1][k]=0;
	  if(currentI+k+1<len1)
	    neighbourhood[0][ALIGNALGNEIGHBOURHOOD+k]=(int)[matrix0 getElementAtRow:toupper([firstNLet0 getSequenceAt:from1+currentI+k+1])-ASCIIA column:toupper(alignArray[1][len])-ASCIIA];
	  else
	    neighbourhood[1][ALIGNALGNEIGHBOURHOOD+k]=0;
	}
      currentNeighbourhood=0.0;
      for(k=0;k<ALIGNALGNEIGHBOURHOOD;k++)
	{
	  currentNeighbourhood+=(double)1/(k+2)*(double)neighbourhood[0][ALIGNALGNEIGHBOURHOOD-k-1];
	  currentNeighbourhood+=(double)1/(k+2)*(double)neighbourhood[0][ALIGNALGNEIGHBOURHOOD+k];
	  currentNeighbourhood+=(double)1/(k+2)*(double)neighbourhood[1][ALIGNALGNEIGHBOURHOOD-k-1];
	  currentNeighbourhood+=(double)1/(k+2)*(double)neighbourhood[1][ALIGNALGNEIGHBOURHOOD+k];
	}
      qualityRow[len]+=(int)currentNeighbourhood;
      len++;
      if(pathMatrix[currentI][currentJ]==0)
	{
	  currentI++;
	  currentJ++;
	}
      else
	if(pathMatrix[currentI][currentJ]==NWNOSTEP)
	  {
	    if(currentI==len1-1)
	      currentI++;
	    if(currentJ==len2-1)
	      currentJ++;
	  }
	else
	  if(pathMatrix[currentI][currentJ]>0)
	    {
	      strcat(alignArray[0],[firstNLet0 getSequenceFrom:from1+currentI+1 to:from1+currentI+pathMatrix[currentI][currentJ]]);
	      for(i=0;i<pathMatrix[currentI][currentJ];i++)
		{
		  alignArray[1][len+i]=ALIGNGAPCHAR;
		  if(i==0)
		    qualityRow[len+i]=-firstGap0;
		  else
		    qualityRow[len+i]=-restGap0;
		}
	      len+=pathMatrix[currentI][currentJ];
	      currentI+=pathMatrix[currentI][currentJ]+1;
	      currentJ++;
	    }
	  else
	    {
	      strcat(alignArray[1],[secondNLet0 getSequenceFrom:from2+currentJ+1 to:from2+currentJ-pathMatrix[currentI][currentJ]]);
	      for(j=0;j<-pathMatrix[currentI][currentJ];j++)
		{
		  alignArray[0][len+j]=ALIGNGAPCHAR;
		  if(j==0)
		    qualityRow[len+j]=-firstGap0;
		  else
		    qualityRow[len+j]=-restGap0;
		}
	      len+=-pathMatrix[currentI][currentJ];
	      currentJ+=-pathMatrix[currentI][currentJ]+1;
	      currentI++;
	    }
    }
  /* 4.3.: Weg vom Endpunkt des gemeinsamen Teils */
  if(currentI<len1)
    {
      strcat(alignArray[0],[firstNLet0 getSequenceFrom:from1+currentI+1 to:from1+len1-1]);
      for(i=0;i<len1-currentI-1;i++)
	{
	  alignArray[1][len+i]=ALIGNGAPCHAR;
	  qualityRow[len+i]=0;
	}
      len+=len1-currentI-1;
    }
  if(currentJ<len2)
    {
      strcat(alignArray[1],[secondNLet0 getSequenceFrom:from2+currentJ+1 to:from2+len2-1]);
      for(j=0;j<len2-currentJ-1;j++)
	{
	  alignArray[0][len+j]=ALIGNGAPCHAR;
	  qualityRow[len+j]=0;
	}
      len+=len2-currentJ-1;
    }
  /* Ende des eigentlichen Algorithmus */
  newAlignment=[Alignment new:alignArray number:2 length:len];
  newAlignment=[newAlignment setQualityRow:qualityRow];
  for(i=0;i<len1;i++)
    {
      free(qualityMatrix[i]);
      free(pathMatrix[i]);
    }
  free(qualityMatrix);
  free(pathMatrix);
  for(i=0;i<2;i++)
    free(neighbourhood[i]);
  free(neighbourhood);
  free(alignArray[0]);
  free(alignArray[1]);
  free(alignArray);
  free(qualityRow);
  free(bestInColumn);
  return newAlignment;
}

@end
