/* Name: alignmat.c                                                          **
**                                                                           **
**                                                                           **
** Funktion: Bereitstellen von Zuordnungsguetematrizen fuer                  **
**           Alignierungsalgorithmen                                         **
**                                                                           **
** Aufruf: -- (Linken der .o Datei zu Programmen)                            **
**                                                                           **
** Probleme:                                                                 **
**                                                                           **
** Erstellt: 23.02.1996                                                      **
** Zuletzt bearbeitet: 02.07.1998                                            */

/* ------------------------------------------------------------------------- */


/* Include-Dateien                                                           */

#include "constant.h"

const int identityMatrix[NUMLETTER][NUMLETTER] =
{
/* A  B  C  D  E  F  G  H  I  J  K  L  M */
  1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, /* A  */
/* N  O  P  Q  R  S  T  U  V  W  X  Y  Z */
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
  0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, /* B  */
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
  0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, /* C  */
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
  0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, /* D  */
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
  0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, /* E  */
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
  0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, /* F  */
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
  0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, /* G  */
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
  0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, /* H  */
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
  0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, /* I  */
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
  0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, /* J  */
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, /* K  */
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, /* L  */
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, /* M  */
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, /* N  */
  1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, /* O  */
  0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, /* P  */
  0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, /* Q  */
  0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, /* R  */
  0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, /* S  */
  0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, /* T  */
  0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, /* U  */
  0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, /* V  */
  0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, /* W  */
  0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, /* X  */
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, /* Y  */
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, /* Z  */
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1}; 

const int standardDNAMatrix[NUMLETTER][NUMLETTER] =
{
/* A  B  C  D  E  F  G  H  I  J  K  L  M */
  4,-2,-3, 1, 0, 0,-3, 1, 0, 0,-2, 0, 1, /* A  */
/* N  O  P  Q  R  S  T  U  V  W  X  Y  Z */
  1, 0, 0, 0, 1,-2,-3,-3, 1, 1, 1,-1, 0, 
 -2, 1, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, /* B  */
  0, 0, 0, 0, 0, 1, 1, 1, 0, 0, 0, 1, 0, 
 -3, 1, 4,-2, 0, 0,-3, 1, 0, 0,-2, 0, 1, /* C  */
  1, 0, 0, 0,-1, 1,-3,-3, 1,-2, 1, 1, 0, 
  1, 0,-2, 1, 0, 0, 1, 0, 0, 0, 1, 0, 0, /* D  */
  0, 0, 0, 0, 1, 0, 1, 1, 0, 1, 0, 0, 0, 
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, /* E  */
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, /* F  */
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
 -3, 1,-3, 1, 0, 0, 4,-2, 0, 0, 1, 0,-2, /* G  */
  1, 0, 0, 0, 1, 1,-3,-3, 1,-2, 1,-1, 0, 
  1, 0, 1, 0, 0, 0,-2, 1, 0, 0, 0, 0, 1, /* H  */
  0, 0, 0, 0, 0, 0, 1, 1, 0, 1, 0, 1, 0, 
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, /* I  */
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, /* J  */
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
 -2, 1,-2, 1, 0, 0, 1, 0, 0, 0, 1, 0, 0, /* K  */
  0, 0, 0, 0, 0, 0, 1, 1, 0, 0, 0, 0, 0, 
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, /* L  */
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
  1, 0, 1, 0, 0, 0,-2, 1, 0, 0, 0, 0, 1, /* M  */
  0, 0, 0, 0, 0, 0,-2,-2, 1, 0, 0, 0, 0, 
  1, 0, 1, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, /* N  */
  1, 0, 0, 0, 0, 0, 1, 1, 0, 0, 1, 0, 0, 
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, /* O  */
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, /* P  */
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, /* Q  */
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
  1, 0,-1, 1, 0, 0, 1, 0, 0, 0, 0, 0, 0, /* R  */
  0, 0, 0, 0, 1, 0,-1,-1, 1, 0, 0,-3, 0, 
 -2, 1, 1, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, /* S  */
  0, 0, 0, 0, 0, 1,-2,-2, 1, 0, 0, 0, 0, 
 -3, 1,-3, 1, 0, 0,-3, 1, 0, 0, 1, 0,-2, /* T  */
  1, 0, 0, 0,-1,-2, 4, 4,-2, 1, 1, 1, 0, 
 -3, 1,-3, 1, 0, 0,-3, 1, 0, 0, 1, 0,-2, /* U  */
  1, 0, 0, 0,-1,-2, 4, 4,-2, 1, 1, 1, 0, 
  1, 0, 1, 0, 0, 0, 1, 0, 0, 0, 0, 0, 1, /* V  */
  0, 0, 0, 0, 1, 1,-2,-2, 1, 0, 0, 0, 0, 
  1, 0,-2, 1, 0, 0,-2, 1, 0, 0, 0, 0, 0, /* W  */
  0, 0, 0, 0, 0, 0, 1, 1, 0, 1, 0, 0, 0, 
  1, 0, 1, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, /* X  */
  1, 0, 0, 0, 0, 0, 1, 1, 0, 0, 1, 0, 0, 
 -1, 1, 1, 0, 0, 0,-1, 1, 0, 0, 0, 0, 0, /* Y  */
  0, 0, 0, 0,-3, 0, 1, 1, 0, 0, 0, 1, 0, 
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, /* Z  */
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}; 

const int pam250Matrix[NUMLETTER][NUMLETTER] =
{
/* A  B  C  D  E  F  G  H  I  J  K  L  M */
  2, 0,-2, 0, 0,-4, 1,-1,-1, 0,-1,-2,-1, /* A  */
/* N  O  P  Q  R  S  T  U  V  W  X  Y  Z */
  0, 0, 1, 0,-2, 1, 1, 0, 0,-6, 0,-3, 0, 
  0, 2,-4, 3, 2,-5, 0, 1,-2, 0, 1,-3,-2, /* B  */
  2, 0,-1, 1,-1, 0, 0, 0,-2,-5, 0,-3, 2, 
 -2,-4,12,-5,-5,-4,-3,-3,-2, 0,-5,-6,-5, /* C  */
 -4, 0,-3,-5,-4, 0,-2, 0,-2,-5, 0, 0,-5, 
  0, 3,-5, 4, 3,-6, 1, 1,-2, 0, 0,-4,-3, /* D  */
  2, 0,-1, 2,-1, 0, 0, 0,-2,-7, 0,-4, 3, 
  0, 2,-5, 3, 4,-5, 0, 1,-2, 0, 0,-3,-2, /* E  */
  1, 0,-1, 2,-1, 0, 0, 0,-2,-7, 0,-4, 3, 
 -4,-5,-4,-6,-5, 9,-5,-2, 1, 0,-5, 2, 0, /* F  */
 -4, 0,-5,-5,-4,-3,-3, 0,-1, 0, 0, 7,-5, 
  1, 0,-3, 1, 0,-5, 5,-2,-3, 0,-2,-4,-3, /* G  */
  0, 0,-1,-1,-3, 1, 0, 0,-1,-7, 0,-5,-1, 
 -1, 1,-3, 1, 1,-2,-2, 6,-2, 0, 0,-2,-2, /* H  */
  2, 0, 0, 3, 2,-1,-1, 0,-2,-3, 0, 0, 2, 
 -1,-2,-2,-2,-2, 1,-3,-2, 5, 0,-2, 2, 2, /* I  */
 -2, 0,-2,-2,-2,-1, 0, 0, 4,-5, 0,-1,-2, 
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, /* J  */
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
 -1, 1,-5, 0, 0,-5,-2, 0,-2, 0, 5,-3, 0, /* K  */
  1, 0,-1, 1, 3, 0, 0, 0,-2,-3, 0,-4, 0, 
 -2,-3,-6,-4,-3, 2,-4,-2, 2, 0,-3, 6, 4, /* L  */
 -3, 0,-3,-2,-3,-3,-2, 0, 2,-2, 0,-1,-3, 
 -1,-2,-5,-3,-2, 0,-3,-2, 2, 0, 0, 4, 6, /* M  */
 -2, 0,-2,-1, 0,-2,-1, 0, 2,-4, 0,-2,-2, 
  0, 2,-4, 2, 1,-4, 0, 2,-2, 0, 1,-3,-2, /* N  */
  2, 0,-1, 1, 0, 1, 0, 0,-2,-4, 0,-2, 1, 
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, /* O  */
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
  1,-1,-3,-1,-1,-5,-1, 0,-2, 0,-1,-3,-2, /* P  */
 -1, 0, 6, 0, 0, 1, 0, 0,-1,-6, 0,-5, 0, 
  0, 1,-5, 2, 2,-5,-1, 3,-2, 0, 1,-2,-1, /* Q  */
  1, 0, 0, 4, 1,-1,-1, 0,-2,-5, 0,-4, 3, 
 -2,-1,-4,-1,-1,-4,-3, 2,-2, 0, 3,-3, 0, /* R  */
  0, 0, 0, 1, 6, 0,-1, 0,-2, 2, 0,-4, 0, 
  1, 0, 0, 0, 0,-3, 1,-1,-1, 0, 0,-3,-2, /* S  */
  1, 0, 1,-1, 0, 2, 1, 0,-1,-2, 0,-3, 0, 
  1, 0,-2, 0, 0,-3, 0,-1, 0, 0, 0,-2,-1, /* T  */
  0, 0, 0,-1,-1, 1, 3, 0, 0,-5, 0,-3,-1, 
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, /* U  */
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
  0,-2,-2,-2,-2,-1,-1,-2, 4, 0,-2, 2, 2, /* V  */
 -2, 0,-1,-2,-2,-1, 0, 0, 4,-6, 0,-2,-2, 
 -6,-5,-8,-7,-7, 0,-7,-3,-5, 0,-3,-2,-4, /* W  */
 -4, 0,-6,-5, 2,-2,-5, 0,-6,17, 0, 0,-6, 
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, /* X  */
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
 -3,-3, 0,-4,-4, 7,-5, 0,-1, 0,-4,-1,-2, /* Y  */
 -2, 0,-5,-4,-4,-3,-3, 0,-2, 0, 0,10,-4, 
  0, 2,-5, 3, 3,-5,-1, 2,-2, 0, 0,-3,-2, /* Z  */
  1, 0, 0, 3, 0, 0,-1, 0,-2,-6, 0,-4, 3}; 
