/* Name: arraydbl.m                                                          **
**                                                                           **
**                                                                           **
** Funktion: Implementation der Klassen ArrayDouble und                      **
**           ArrayDoubleWStatistics, die ein Feld reeller Zahlen (mit        **
**           Statistik) bereitstellen                                        **
**                                                                           **
** Aufruf: -- (Linken mit anderen .o-Dateien moeglich)                       **
**                                                                           **
** Probleme:                                                                 **
**                                                                           **
** Erstellt: 22.07.1995                                                      **
** Zuletzt bearbeitet: 03.05.1996                                            */

/* ------------------------------------------------------------------------- */


/* Include-Dateien                                                           */

#include "arraydbl.h"

#include "error.h"
#include "constant.h"

#include <math.h>
#include <stdlib.h>

/* Klasse: ArrayDouble                                                       */

@implementation ArrayDouble

+ new
{
  return [[self alloc] init];
}

+ new:(int)size0 array:(double*)array0
{
  return [[self alloc] init:size0 array:array0];
}

+ alloc
{
  ArrayDouble *newObj;
  newObj=[super alloc];
  newObj->array=(double*)malloc(MINARRAYSIZE*sizeof(double));
  if(!newObj->array)
    [ErrorHandler error:ERRMEMNOTENOUGH];
  return newObj;
}

- init
{
  self=[super init];
  size=0;
  return self;
}

- init:(int)size0 array:(double*)array0
{
  int i;
  self=[super init];
  size=size0;
  if(size<0)
    {
      [ErrorHandler error:ERROBJINIT];
      return self;
    }
  array=(double*)realloc(array,size*sizeof(double));
  if(!array)
    {
      [ErrorHandler error:ERRMEMNOTENOUGH];
      return self;
    }
  for(i=0;i<size;i++)
    if(array0+i)
      array[i]=array0[i];
    else
      {
        [ErrorHandler error:ERROBJINIT];
        return self;
      }
  return self;
}

- append:(double)value0
{
  array=(double*)realloc(array,(size+1)*sizeof(double));
  array[size]=value0;
  size++;
  return self;
}

- insertAt:(int)pos0 value:(double)value0
{
  int i;
  array=(double*)realloc(array,(size+1)*sizeof(double));
  for(i=size;i>pos0;i--)
    array[i]=array[i-1];
  array[pos0]=value0;
  size++;
  return self;
}

- deleteAt:(int)pos0
{
  int i;
  for(i=pos0;i<size-1;i--)
    array[i]=array[i+1];
  array=(double*)realloc(array,(size-1)*sizeof(double));
  size--;
  return self;
}

- (int)getSize
{
  return size;
}

- (double)getElementAt:(int)pos0
{
  if(pos0>=0 && pos0<size)
    return array[pos0];
  else
    return DEFDOUBLE;
}

- free
{
  free(array);
  return [super free];
}

@end

/* Klasse: ArrayDoubleWStatistics                                            */

@implementation ArrayDoubleWStatistics

+ updateStatistics:(ArrayDoubleWStatistics*)array0
{
  int i;
  if(!array0->updated)
    {
      array0->sum=0.0;
      array0->mean=0.0;
      array0->stdDeviation=0.0;
      if([array0 getSize]>0)
	{
	  for(i=0;i<[array0 getSize];i++)
	    array0->sum+=[array0 getElementAt:i];
	  array0->mean=array0->sum/(double)[array0 getSize];
	  if([array0 getSize]>1)
	    {
	      for(i=0;i<[array0 getSize];i++)
		array0->stdDeviation+=pow([array0 getElementAt:i]-array0->mean,2.0);
	      array0->stdDeviation/=(double)[array0 getSize]-1;
	      array0->stdDeviation=sqrt(array0->stdDeviation);
	    }
	}
    }
  array0->updated=YES;
  return array0;
}

- init
{
  size=0;
  updated=NO;
  return self;
}

- init:(int)size0 array:(double*)array0
{
  int i;
  size=size0;
  if(size<0)
    {
      [ErrorHandler error:ERROBJINIT];
      return self;
    }
  array=(double*)realloc(array,size*sizeof(double));
  if(!array)
    {
      [ErrorHandler error:ERRMEMNOTENOUGH];
      return self;
    }
  for(i=0;i<size;i++)
    if(array0+i)
      array[i]=array0[i];
    else
      {
        [ErrorHandler error:ERROBJINIT];
        return self;
      }
  return self;
  updated=NO;
}

- append:(double)value0
{
  array=(double*)realloc(array,(size+1)*sizeof(double));
  array[size]=value0;
  size++;
  updated=NO;
  return self;
}

- insertAt:(int)pos0 value:(double)value0
{
  int i;
  array=(double*)realloc(array,(size+1)*sizeof(double));
  for(i=size;i>pos0;i--)
    array[i]=array[i-1];
  array[pos0]=value0;
  size++;
  updated=NO;
  return self;
}

- deleteAt:(int)pos0
{
  int i;
  for(i=pos0;i<size-1;i--)
    array[i]=array[i+1];
  array=(double*)realloc(array,(size-1)*sizeof(double));
  size--;
  updated=NO;
  return self;
}

- (int)getFrequency:(double)value0
{
  int frequency, i;
  frequency=0;
  for(i=0;i<size;i++)
    if(array[i]==value0)
      frequency++;
  return frequency;
}

- (double)getSum
{
  int i;
  if(!updated)
    [[self class] updateStatistics:self];
  return sum;
}

- (double)getMean
{
  int i;
  if(size==0)
    return DEFDOUBLE;
  if(!updated)
    [[self class] updateStatistics:self];
  return mean;
}

- (double)getStdDeviation
{
  int i;
  if(size==0)
    return DEFDOUBLE;
  if(!updated)
    [[self class] updateStatistics:self];
  updated=YES;
  return stdDeviation;
}

@end
