/* Name: dnanlet.m                                                           **
**                                                                           **
**                                                                           **
** Funktion: Implementation der Klasse DNANLet, die einen Sequenzabschnitt   **
**           einer DNA beschreibt (Ableitung der Klasse NLet)                **
**                                                                           **
** Aufruf: -- (Linken mit anderen .o-Dateien moeglich)                       **
**                                                                           **
** Probleme:                                                                 **
**                                                                           **
** Erstellt: 28.07.1995                                                      **
** Zuletzt bearbeitet: 17.11.1996                                            */

/* ------------------------------------------------------------------------- */

/* Include-Dateien                                                           */

#include "dnanlet.h"

#include "protnlet.h"
#include "error.h"
#include "constant.h"

#include <ctype.h>
#include <string.h>
#include <math.h>
#include <stdlib.h>

/* Klasse: DNANLet                                                           */

@implementation DNANLet

char complementaryNucleotides[]="tvgh  cd  m kn   ysaabwnr ";

+ (BOOL)isInDNAAlphabet:(char)char0
{
  switch(tolower(char0))
    {
    case 'a' : case 'c' : case 'g' : case 't' : case 'u' :
    case 'm' : case 'r' : case 'w' : case 's' : case 'y' : case 'k' :
    case 'v' : case 'h' : case 'd' : case 'b' :
    case 'n' : case 'x' : 
      return YES;
    default : 
      return NO;
    }
}

+ (BOOL)isDNANucleotide:(char)char0
{
  switch(tolower(char0))
    {
    case 'a' : case 'c' : case 'g' : case 't' : 
      return YES;
    default : 
      return NO;
    }
}

+ (BOOL)isRNANucleotide:(char)char0
{
  switch(tolower(char0))
    {
    case 'a' : case 'c' : case 'g' : case 'u' : 
      return YES;
    default : 
      return NO;
    }
}

+ (char)complementaryNucleotide:(char)base0
{
  if([self isInDNAAlphabet:base0])
    return complementaryNucleotides[tolower(base0)-ASCIIa];
  else
    return DEFNLETELEMENT;
}

+ (char)minimumCommonExtension:(char)firstBase0 and:(char)secondBase0
{
  char minCommonExt;
  minCommonExt=DEFNLETELEMENT;
  if([self isInDNAAlphabet:firstBase0] && [self isInDNAAlphabet:secondBase0])
    if(tolower(firstBase0)=='n' || tolower(secondBase0)=='n' || tolower(firstBase0)=='x' || tolower(secondBase0)=='x')
      minCommonExt='n';
    else
      switch(tolower(firstBase0))
        {
	case 'v' :
	  switch(tolower(secondBase0))
	    {
	    case 'a' : case 'c' : case 'g':
	    case 'm' : case 'r' : case 's' :
	    case 'v' : 
	      minCommonExt='v';
	      break;
	    default : 
	      minCommonExt='n';
	      break;
	    }
	  break;
	case 'h' :
	  switch(tolower(secondBase0))
	    {
	    case 'a' : case 'c' : case 't':
	    case 'm' : case 'y' : case 'w' :
	    case 'h' : 
	      minCommonExt='h';
	      break;
	    default : 
	      minCommonExt='n';
	      break;
	    }
	  break;
	case 'd' :
	  switch(tolower(secondBase0))
	    {
	    case 'a' : case 'g' : case 't':
	    case 'r' : case 'k' : case 'w' :
	    case 'd' : 
	      minCommonExt='d';
	      break;
	    default : 
	      minCommonExt='n';
	      break;
	    }
	  break;
	case 'b' :
	  switch(tolower(secondBase0))
	    {
	    case 'c' : case 'g' : case 't':
	    case 'y' : case 'k' : case 's' :
	    case 'd' : 
	      minCommonExt='d';
	      break;
	    default : 
	      minCommonExt='n';
	      break;
	    }
	  break;
	case 'm' :
	  switch(tolower(secondBase0))
	    {
	    case 'a' : case 'c' : case 'm' :
	      minCommonExt='m';
	      break;
	    case 'g' : case 'v' : case 'r' : case 's':
	      minCommonExt='v';
	      break;
	    case 't' : case 'u' : case 'h' : case 'y' : case 'w':
	      minCommonExt='h';
	      break;
	    default : 
	      minCommonExt='n';
	      break;
	    }
	  break;
	case 'r' :
	  switch(tolower(secondBase0))
	    {
	    case 'a' : case 'g' : case 'r' :
	      minCommonExt='r';
	      break;
	    case 'c' : case 'v' : case 'm' : case 's':
	      minCommonExt='v';
	      break;
	    case 't' : case 'u' : case 'd' : case 'k' : case 'w':
	      minCommonExt='d';
	      break;
	    default : 
	      minCommonExt='n';
	      break;
	    }
	  break;
	case 'w' :
	  switch(tolower(secondBase0))
	    {
	    case 'a' : case 't' : case 'u' : case 'w' :
	      minCommonExt='w';
	      break;
	    case 'c' : case 'h' : case 'm' : case 'y':
	      minCommonExt='h';
	      break;
	    case 'g' : case 'd' : case 'r' : case 'k':
	      minCommonExt='d';
	      break;
	    default : 
	      minCommonExt='n';
	      break;
	    }
	  break;
	case 's' :
	  switch(tolower(secondBase0))
	    {
	    case 'c' : case 'g' : case 's' :
	      minCommonExt='s';
	      break;
	    case 'a' : case 'v' : case 'm' : case 'r':
	      minCommonExt='v';
	      break;
	    case 't' : case 'u' : case 'b' : case 'y' : case 'k':
	      minCommonExt='b';
	      break;
	    default : 
	      minCommonExt='n';
	      break;
	    }
	  break;
	case 'y' :
	  switch(tolower(secondBase0))
	    {
	    case 'c' : case 't' : case 'u' : case 'y' :
	      minCommonExt='y';
	      break;
	    case 'a' : case 'h' : case 'm' : case 'w':
	      minCommonExt='h';
	      break;
	    case 'g' : case 'b' : case 's' : case 'k':
	      minCommonExt='b';
	      break;
	    default : 
	      minCommonExt='n';
	      break;
	    }
	  break;
	case 'k' :
	  switch(tolower(secondBase0))
	    {
	    case 'g' : case 't' : case 'u' : case 'k' :
	      minCommonExt='k';
	      break;
	    case 'a' : case 'd' : case 'r' : case 'w':
	      minCommonExt='d';
	      break;
	    case 'c' : case 'b' : case 's' : case 'y':
	      minCommonExt='b';
	      break;
	    default : 
	      minCommonExt='n';
	      break;
	    }
	  break;
	case 'a' :
	  switch(tolower(secondBase0))
	    {
	    case 'a' : 
	      minCommonExt='a';
	      break;
	    case 'c' : case 'm' :
	      minCommonExt='m';
	      break;
	    case 'g' : case 'r' :
	      minCommonExt='r';
	      break;
	    case 't' : case 'u' : case 'w' :
	      minCommonExt='w';
	      break;
	    case 's' : case 'v' :
	      minCommonExt='v';
	      break;
	    case 'y' : case 'h' :
	      minCommonExt='h';
	      break;
	    case 'k' : case 'd' :
	      minCommonExt='d';
	      break;
	    default : 
	      minCommonExt='n';
	      break;
	    }
	  break;
	case 'c' :
	  switch(tolower(secondBase0))
	    {
	    case 'c' : 
	      minCommonExt='c';
	      break;
	    case 'a' : case 'm' :
	      minCommonExt='m';
	      break;
	    case 'g' : case 's' :
	      minCommonExt='s';
	      break;
	    case 't' : case 'u' : case 'y' :
	      minCommonExt='y';
	      break;
	    case 'r' : case 'v' :
	      minCommonExt='v';
	      break;
	    case 'w' : case 'h' :
	      minCommonExt='h';
	      break;
	    case 'k' : case 'b' :
	      minCommonExt='b';
	      break;
	    default : 
	      minCommonExt='n';
	      break;
	    }
	  break;
	case 'g' :
	  switch(tolower(secondBase0))
	    {
	    case 'g' : 
	      minCommonExt='g';
	      break;
	    case 'a' : case 'r' :
	      minCommonExt='r';
	      break;
	    case 'c' : case 's' :
	      minCommonExt='s';
	      break;
	    case 't' : case 'u' : case 'k' :
	      minCommonExt='k';
	      break;
	    case 'm' : case 'v' :
	      minCommonExt='v';
	      break;
	    case 'w' : case 'd' :
	      minCommonExt='d';
	      break;
	    case 'y' : case 'b' :
	      minCommonExt='b';
	      break;
	    default : 
	      minCommonExt='n';
	      break;
	    }
	  break;
	case 't' : case 'u' :
	  switch(tolower(secondBase0))
	    {
	    case 't' : case 'u' : 
	      minCommonExt='t';
	      break;
	    case 'a' : case 'w' :
	      minCommonExt='w';
	      break;
	    case 'c' : case 'y' :
	      minCommonExt='y';
	      break;
	    case 'g' : case 'k' :
	      minCommonExt='k';
	      break;
	    case 'm' : case 'h' :
	      minCommonExt='h';
	      break;
	    case 'r' : case 'd' :
	      minCommonExt='d';
	      break;
	    case 's' : case 'b' :
	      minCommonExt='b';
	      break;
	    default : 
	      minCommonExt='n';
	      break;
	    }
	  break;
	default : 
	  break;
        }
  return minCommonExt;
}

+ (char)translateTriplet:(char*)triplet0
{
  char translTriplet;
  translTriplet=DEFNLETELEMENT;
  if(strlen(triplet0)>=3)
    switch(tolower(triplet0[0]))
      {
      case 'a' :
	switch(tolower(triplet0[1]))
	  {
	  case 'a' :
	    switch(tolower(triplet0[2]))
	      {
	      case 'a' : case 'g' :
		translTriplet='K';
		break;
	      case 'c' : case 't' : case 'u' :
		translTriplet='N';
		break;
	      default :
		break;
	      };
	    break;
	  case 'c' :
	    switch(tolower(triplet0[2]))
	      {
	      case 'a' : case 'c' :
	      case 'g' : case 't' : case 'u' :
		translTriplet='T';
		break;
	      default :
		break;
	      };
	    break;
	  case 'g' :
	    switch(tolower(triplet0[2]))
	      {
	      case 'a' : case 'g' :
		translTriplet='R';
		break;
	      case 'c' : case 't' : case 'u' :
		translTriplet='S';
		break;
	      default :
		break;
	      };
	    break;
	  case 't' : case 'u' :
	    switch(tolower(triplet0[2]))
	      {
	      case 'a' : case 'c' : case 't' : case 'u' :
		translTriplet='I';
		break;
	      case 'g' : 
		translTriplet='M';
		break;
	      default :
		break;
	      };
	    break;
	  default :
	    break;
	  };
	break;
      case 'c' :
	switch(tolower(triplet0[1]))
	  {
	  case 'a' :
	    switch(tolower(triplet0[2]))
	      {
	      case 'a' : case 'g' :
		translTriplet='Q';
		break;
	      case 'c' : case 't' : case 'u' :
		translTriplet='H';
		break;
	      default :
		break;
	      };
	    break;
	  case 'c' :
	    switch(tolower(triplet0[2]))
	      {
	      case 'a' : case 'c' :
	      case 'g' : case 't' : case 'u' :
		translTriplet='P';
		break;
	      default :
		break;
	      };
	    break;
	  case 'g' :
	    switch(tolower(triplet0[2]))
	      {
	      case 'a' : case 'c' :
	      case 'g' : case 't' : case 'u' :
		translTriplet='R';
		break;
	      default :
		break;
	      };
	    break;
	  case 't' : case 'u' :
	    switch(tolower(triplet0[2]))
	      {
	      case 'a' : case 'c' :
	      case 'g' : case 't' : case 'u' :
		translTriplet='L';
		break;
	      default :
		break;
	      };
	    break;
	  default :
	    break;
	  };
	break;
      case 'g' :
	switch(tolower(triplet0[1]))
	  {
	  case 'a' :
	    switch(tolower(triplet0[2]))
	      {
	      case 'a' : case 'g' :
		translTriplet='E';
		break;
	      case 'c' : case 't' : case 'u' :
		translTriplet='D';
		break;
	      default :
		break;
	      };
	    break;
	  case 'c' :
	    switch(tolower(triplet0[2]))
	      {
	      case 'a' : case 'c' :
	      case 'g' : case 't' : case 'u' :
		translTriplet='A';
		break;
	      default :
		break;
	      };
	    break;
	  case 'g' :
	    switch(tolower(triplet0[2]))
	      {
	      case 'a' : case 'c' :
	      case 'g' : case 't' : case 'u' :
		translTriplet='G';
		break;
	      default :
		break;
	      };
	    break;
	  case 't' : case 'u' :
	    switch(tolower(triplet0[2]))
	      {
	      case 'a' : case 'c' :
	      case 'g' : case 't' : case 'u' :
		translTriplet='V';
		break;
	      default :
		break;
	      };
	    break;
	  default :
	    break;
	  };
	break;
      case 't' : case 'u' :
	switch(tolower(triplet0[1]))
	  {
	  case 'a' :
	    switch(tolower(triplet0[2]))
	      {
	      case 'a' : case 'g' :
		translTriplet='*';
		break;
	      case 'c' : case 't' : case 'u' :
		translTriplet='Y';
		break;
	      default :
		break;
	      };
	    break;
	  case 'c' :
	    switch(tolower(triplet0[2]))
	      {
	      case 'a' : case 'c' :
	      case 'g' : case 't' : case 'u' :
		translTriplet='S';
		break;
	      default :
		break;
	      };
	    break;
	  case 'g' :
	    switch(tolower(triplet0[2]))
	      {
	      case 'a' :
		translTriplet='*';
		break;
	      case 'c' : case 't' : case 'u' :
		translTriplet='C';
		break;
	      case 'g' :
		translTriplet='W';
		break;
	      default :
		break;
	      };
	    break;
	  case 't' : case 'u' :
	    switch(tolower(triplet0[2]))
	      {
	      case 'a' : case 'g' :
		translTriplet='L';
		break;
	      case 'c' : case 't' : case 'u' :
		translTriplet='F';
		break;
	      default :
		break;
	      };
	    break;
	  default :
	    break;
	  };
	break;
      default :
	break;
      }
  return translTriplet;
}

+ (DNANLet*)antisenseStrand:(DNANLet*)dnaNLet0
{
  return [self antisenseStrand:dnaNLet0 from:0 to:[dnaNLet0 getLength]-1];
}

+ (DNANLet*)antisenseStrand:(DNANLet*)dnaNLet0 from:(long)i0 to:(long)j0;
{
  DNANLet *antisenseNLet;
  long from, to;
  char *antisenseSequence;
  long i, j;
  if(![dnaNLet0 isKindOf:[DNANLet class]])
    {
      [ErrorHandler error:ERRMETHODWRONGOBJ];
      return nil;
    }
  if(i0<0)
    from=0;
  else
    from=i0;
  if(j0>=[dnaNLet0 getLength])
    to=[dnaNLet0 getLength]-1;
  else
    to=j0;
  antisenseSequence=(char*)malloc((to-from+2)*sizeof(char));
  if(!antisenseSequence)
    {
      [ErrorHandler error:ERRMEMNOTENOUGH];      
      return nil;
    }
  strcpy(antisenseSequence,"");
  i=to;
  j=0;
  while(i>=from)
    antisenseSequence[j++]=[self complementaryNucleotide:[dnaNLet0 getSequenceAt:i--]];
  antisenseSequence[j]='\0';
  antisenseNLet=[DNANLet new:antisenseSequence];
  free(antisenseSequence);
  return antisenseNLet;
}

+ (ProteinNLet*)translate:(DNANLet*)dnaNLet0
{
  return [self translate:dnaNLet0 from:0 to:[dnaNLet0 getLength]-1 withOffset:DEFDNANLETOFFSET];
}

+ (ProteinNLet*)translate:(DNANLet*)dnaNLet0 from:(long)i0 to:(long)j0
{
  return [self translate:dnaNLet0 from:i0 to:j0 withOffset:DEFDNANLETOFFSET];
}

+ (ProteinNLet*)translate:(DNANLet*)dnaNLet0 withOffset:(int)offset0
{
  return [self translate:dnaNLet0 from:0 to:[dnaNLet0 getLength]-1 withOffset:offset0];
}

+ (ProteinNLet*)translate:(DNANLet*)dnaNLet0 from:(long)i0 to:(long)j0 withOffset:(int)offset0
{
  ProteinNLet *translNLet;
  long from, to;
  int offset;
  char *translSequence;
  char *curTriplet;
  long i, j;
  if(![dnaNLet0 isKindOf:[DNANLet class]] || ![dnaNLet0 isUnambiguous])
    {
      [ErrorHandler error:ERRMETHODWRONGOBJ];
      return nil;
    }
  if(i0<0)
    from=0;
  else
    from=i0;
  if(j0>=[dnaNLet0 getLength])
    to=[dnaNLet0 getLength]-1;
  else
    to=j0;
  offset=offset0%3;
  translSequence=(char*)malloc(((to-from+1)/3+1)*sizeof(char));
  curTriplet=(char*)malloc(3*sizeof(char));
  if(!translSequence || !curTriplet)
    {
      [ErrorHandler error:ERRMEMNOTENOUGH];      
      return nil;
    }
  strcpy(translSequence,"");
  i=from+offset;
  j=0;
  while(i<to-1)
    {
      strcpy(curTriplet,[dnaNLet0 getSequenceFrom:i to:i+2]);
      translSequence[j++]=[self translateTriplet:curTriplet];
      i+=3;
    }
  translSequence[j]='\0';
  translNLet=[ProteinNLet new:translSequence];
  free(translSequence);
  free(curTriplet);
  return translNLet;
}

+ (DNANLet*)tripletReverse:(DNANLet*)dnaNLet0
{
  return [self tripletReverse:dnaNLet0 from:0 to:[dnaNLet0 getLength]-1 withOffset:DEFDNANLETOFFSET];
}

+ (DNANLet*)tripletReverse:(DNANLet*)dnaNLet0 from:(long)i0 to:(long)j0
{
  return [self tripletReverse:dnaNLet0 from:i0 to:j0 withOffset:DEFDNANLETOFFSET];
}

+ (DNANLet*)tripletReverse:(DNANLet*)dnaNLet0 withOffset:(int)offset0
{
  return [self tripletReverse:dnaNLet0 from:0 to:[dnaNLet0 getLength]-1 withOffset:offset0];
}

+ (DNANLet*)tripletReverse:(DNANLet*)dnaNLet0 from:(long)i0 to:(long)j0 withOffset:(int)offset0
{
  DNANLet *reversedNLet;
  long from, to;
  int offset;
  char *reversedSequence;
  long i, j;
  if(![dnaNLet0 isKindOf:[DNANLet class]])
    {
      [ErrorHandler error:ERRMETHODWRONGOBJ];
      return nil;
    }
  if(i0<0)
    from=0;
  else
    from=i0;
  if(j0>=[dnaNLet0 getLength])
    to=[dnaNLet0 getLength]-1;
  else
    to=j0;
  offset=offset0%3;
  reversedSequence=(char*)malloc((to-from+2)*sizeof(char));
  if(!reversedSequence)
    {
      [ErrorHandler error:ERRMEMNOTENOUGH];      
      return nil;
    }
  strcpy(reversedSequence,"");
  i=from+offset;
  j=0;
  while(i<to-1)
    {
      reversedSequence[j++]=tolower([dnaNLet0 getSequenceAt:i+2]);
      reversedSequence[j++]=tolower([dnaNLet0 getSequenceAt:i+1]);
      reversedSequence[j++]=tolower([dnaNLet0 getSequenceAt:i]);
      i+=3;
    }
  reversedSequence[j]='\0';
  reversedNLet=[DNANLet new:reversedSequence];
  free(reversedSequence);
  return reversedNLet;
}

+ (DNANLet*)minimumInformation:(DNANLet*)dnaNLet0
{
  return [self minimumInformation:dnaNLet0 from:0 to:[dnaNLet0 getLength]-1];
}

+ (DNANLet*)minimumInformation:(DNANLet*)dnaNLet0 from:(long)i0 to:(long)j0
{
  DNANLet *minInformation;
  ProteinNLet *transl1, *transl2, *transl3;
  DNANLet *retransl1, *retransl2, *retransl3;
  long from, to;
  char *minInfSequence;
  long i, j;
  if(![dnaNLet0 isKindOf:[DNANLet class]])
    {
      [ErrorHandler error:ERRMETHODWRONGOBJ];
      return nil;
    }
  if(i0<0)
    from=0;
  else
    from=i0;
  if(j0>=[dnaNLet0 getLength])
    to=[dnaNLet0 getLength]-1;
  else
    to=j0;
  minInfSequence=(char*)malloc((to-from-2)*sizeof(char));
  if(!minInfSequence)
    {
      [ErrorHandler error:ERRMEMNOTENOUGH];      
      return nil;
    }
  transl1=[self translate:dnaNLet0 from:from to:to withOffset:0];
  transl2=[self translate:dnaNLet0 from:from to:to withOffset:1];
  transl3=[self translate:dnaNLet0 from:from to:to withOffset:2];
  retransl1=[ProteinNLet retranslate:transl1];
  retransl2=[ProteinNLet retranslate:transl2];
  retransl3=[ProteinNLet retranslate:transl3];
  if(!retransl1 || !retransl2 || !retransl3)
    {
      [ErrorHandler error:ERRMEMNOTENOUGH];      
      free(minInfSequence);
      return nil;
    }
  strcpy(minInfSequence,"");
  i=j=0;
  do
    minInfSequence[j++]=[self minimumCommonExtension:[retransl1 getSequenceAt:i+2] and:[self minimumCommonExtension:[retransl2 getSequenceAt:i+1] and:[retransl3 getSequenceAt:i]]];
  while(++i<to-from-3);
  minInfSequence[j]='\0';
  minInformation=[DNANLet new:minInfSequence];
  free(minInfSequence);
  return minInformation;
}

- init:(char*)seq0
{
  BOOL validSeq;
  long i;
  self=[super init];
  validSeq=YES;
  if(seq0)
    {
      for(i=0;i<strlen(seq0);i++)
	if(validSeq && ![[self class] isInDNAAlphabet:seq0[i]])
	    validSeq=NO;
      if(validSeq)
	{
	  length=strlen(seq0);
	  sequence=(char*)realloc(sequence,(length+1)*sizeof(char));
	  if(length && !sequence)
	    {
	      [ErrorHandler error:ERRMEMNOTENOUGH];
	      return self;
	    }
	  for(i=0;i<strlen(seq0);i++)
	    sequence[i]=tolower(seq0[i]);
	  sequence[i]='\0';
	}
      else
	{
	  length=DEFNLETLEN;
	  sequence=(char*)realloc(sequence,(length+1)*sizeof(char));
	  if(length && !sequence)
	    {
	      [ErrorHandler error:ERRMEMNOTENOUGH];
	      return self;
	    }
	  strcpy(sequence,DEFNLETSEQUENCE);
	}
      for(i=0;i<SIZEASCII;i++)
	composition[i]=0;
      for(i=0;i<length;i++)
	composition[sequence[i]]++;
      informationDensity=0.0;
      if(length)
	{
	  for(i=0;i<length;i++)
	    switch(tolower(sequence[i]))
	      {
	      case 'a' : case 'c' : case 'g' : case 't' : case 'u' :
		informationDensity+=log(0.25)/log(4.0);
		break;
	      case 'm' : case 'r' : case 'w' : case 's' : case 'y' : case 'k' :
		informationDensity+=log(0.5)/log(4.0);
		break;
	      case 'v' : case 'h' : case 'd' : case 'b' :
		informationDensity+=log(0.75)/log(4.0);
		break;
	      default : 
		break;
	      }
	  informationDensity/=-length;
	}
      if(informationDensity==1.0)
	unambiguous=YES;
      else
	unambiguous=NO;
    }
  else
    {
      length=DEFNLETLEN;
      for(i=0;i<SIZEASCII;i++)
	composition[i]=0;
      for(i=0;i<length;i++)
	composition[sequence[i]]++;
      informationDensity=0.0;
    }
  return self;
}

- setSequence: (char*) seq0
{
  BOOL validSeq;
  long i;
  if(seq0)
    {
      validSeq=YES;
      for(i=0;i<strlen(seq0);i++)
	if(validSeq && ![[self class] isInDNAAlphabet:seq0[i]])
	  validSeq=NO;
      if(validSeq)
	{
	  length=strlen(seq0);
	  sequence=realloc(sequence,(length+1)*sizeof(char));
	  if(length && !sequence)
	    {
	      [ErrorHandler error:ERRMEMNOTENOUGH];
	      return self;
	    }
	  for(i=0;i<strlen(seq0);i++)
	    sequence[i]=tolower(seq0[i]);
	  sequence[i]='\0';
	}
      for(i=0;i<SIZEASCII;i++)
	composition[i]=0;
      for(i=0;i<length;i++)
	composition[sequence[i]]++;
      informationDensity=0.0;
      for(i=0;i<length;i++)
	switch(tolower(sequence[i]))
	  {
	  case 'a' : case 'c' : case 'g' : case 't' : case 'u' :
	    informationDensity+=log(0.25)/log(4.0);
	    break;
	  case 'm' : case 'r' : case 'w' : case 's' : case 'y' : case 'k' :
	    informationDensity+=log(0.5)/log(4.0);
	    break;
	  case 'v' : case 'h' : case 'd' : case 'b' :
	    informationDensity+=log(0.75)/log(4.0);
	    break;
	  default : 
	    break;
	  }
      informationDensity/=-length;
      if(informationDensity==1.0)
	unambiguous=YES;
      else
	unambiguous=NO;
    }
  return self;
}

- (BOOL)isUnambiguous
{
  return unambiguous; 
}

- (double)getInformationDensity
{
  return informationDensity;
}

@end
