/* Name: protnlet.m                                                          **
**                                                                           **
**                                                                           **
** Funktion: Implementation der Klasse ProteinNLet, die einen Sequenz-       **
**           abschnitt einer DNA beschreibt (Ableitung der Klasse NLet)      **
**                                                                           **
** Aufruf: -- (Linken mit anderen .o-Dateien moeglich)                       **
**                                                                           **
** Probleme:                                                                 **
**                                                                           **
** Erstellt: 28.07.1995                                                      **
** Zuletzt bearbeitet: 01.04.1997                                            */

/* ------------------------------------------------------------------------- */

/* Include-Dateien                                                           */

#include "protnlet.h"

#include "dnanlet.h"
#include "error.h"
#include "constant.h"

#include <ctype.h>
#include <string.h>
#include <stdlib.h>

/* Klasse: ProteinNLet                                                       */

@implementation ProteinNLet

+ (BOOL)isInProteinAlphabet:(char)char0
{
  return ([self isAminoAcid:char0] || [self isComposedAminoAcid:char0] || [self isUnknownAminoAcid:char0] || [self isStop:char0]);
}

+ (BOOL)isAminoAcid:(char)char0
{
  switch(toupper(char0))
    {
    case 'A' : case 'C' : case 'D' : case 'E' :
    case 'F' : case 'G' : case 'H' : case 'I' :
    case 'K' : case 'L' : case 'M' : case 'N' :
    case 'P' : case 'Q' : case 'R' : case 'S' :
    case 'T' : case 'V' : case 'W' : case 'Y' : 
      return YES;
    default : 
      return NO;
    }
}

+ (BOOL)isComposedAminoAcid:(char)char0
{
  switch(toupper(char0))
    {
    case 'B' : case 'Z' : 
      return YES;
    default : 
      return NO;
    }
}

+ (BOOL)isUnknownAminoAcid:(char)char0
{
  switch(toupper(char0))
    {
    case 'X' : 
      return YES;
    default : 
      return NO;
    }
}

+ (BOOL)isStop:(char)char0
{
  if(char0==PROTNLETSTOP)
    return YES;
  else
    return NO;
}


+ (char*)retranslateAminoAcid:(char)aminoAcid0
{
  char *retranslAminoAcid;
  retranslAminoAcid=(char*)malloc(4*sizeof(char));
  if(!retranslAminoAcid)
    {
      [ErrorHandler error:ERRMEMNOTENOUGH];      
      return DEFNLETSEQUENCE;
    }
  strcpy(retranslAminoAcid,DEFNLETSEQUENCE);
  switch(toupper(aminoAcid0))
    {
    case 'A' :
      retranslAminoAcid[0]='g';
      retranslAminoAcid[1]='c';
      retranslAminoAcid[2]='n';
      break;
    case 'C' :
      retranslAminoAcid[0]='t';
      retranslAminoAcid[1]='g';
      retranslAminoAcid[2]='h';
      break;
    case 'D' :
      retranslAminoAcid[0]='g';
      retranslAminoAcid[1]='a';
      retranslAminoAcid[2]='y';
      break;
    case 'E' :
      retranslAminoAcid[0]='g';
      retranslAminoAcid[1]='a';
      retranslAminoAcid[2]='r';
      break;
    case 'F' :
      retranslAminoAcid[0]='t';
      retranslAminoAcid[1]='t';
      retranslAminoAcid[2]='y';
      break;
    case 'G' :
      retranslAminoAcid[0]='g';
      retranslAminoAcid[1]='g';
      retranslAminoAcid[2]='n';
      break;
    case 'H' :
      retranslAminoAcid[0]='c';
      retranslAminoAcid[1]='a';
      retranslAminoAcid[2]='y';
      break;
    case 'I' :
      retranslAminoAcid[0]='a';
      retranslAminoAcid[1]='t';
      retranslAminoAcid[2]='h';
      break;
    case 'K' :
      retranslAminoAcid[0]='a';
      retranslAminoAcid[1]='a';
      retranslAminoAcid[2]='r';
      break;
    case 'L' :
      retranslAminoAcid[0]='y';
      retranslAminoAcid[1]='t';
      retranslAminoAcid[2]='n';
      break;
    case 'M' :
      retranslAminoAcid[0]='a';
      retranslAminoAcid[1]='t';
      retranslAminoAcid[2]='g';
      break;
    case 'N' :
      retranslAminoAcid[0]='a';
      retranslAminoAcid[1]='a';
      retranslAminoAcid[2]='y';
      break;
    case 'P' :
      retranslAminoAcid[0]='c';
      retranslAminoAcid[1]='c';
      retranslAminoAcid[2]='n';
      break;
    case 'Q' :
      retranslAminoAcid[0]='c';
      retranslAminoAcid[1]='a';
      retranslAminoAcid[2]='r';
      break;
    case 'R' :
      retranslAminoAcid[0]='m';
      retranslAminoAcid[1]='g';
      retranslAminoAcid[2]='n';
      break;
    case 'S' :
      retranslAminoAcid[0]='w';
      retranslAminoAcid[1]='s';
      retranslAminoAcid[2]='n';
      break;
    case 'T' :
      retranslAminoAcid[0]='a';
      retranslAminoAcid[1]='c';
      retranslAminoAcid[2]='n';
      break;
    case 'V' :
      retranslAminoAcid[0]='g';
      retranslAminoAcid[1]='t';
      retranslAminoAcid[2]='n';
      break;
    case 'W' :
      retranslAminoAcid[0]='t';
      retranslAminoAcid[1]='g';
      retranslAminoAcid[2]='g';
      break;
    case 'Y' :
      retranslAminoAcid[0]='t';
      retranslAminoAcid[1]='a';
      retranslAminoAcid[2]='y';
      break;
    case '*' :
      retranslAminoAcid[0]='t';
      retranslAminoAcid[1]='a';
      retranslAminoAcid[2]='r';
      break;
    default :
      break;
    }
  retranslAminoAcid[3]='\0';
  return retranslAminoAcid;
}

+ (DNANLet*)retranslate:(ProteinNLet*)proteinNLet0
{
  return [self retranslate:proteinNLet0 from:0 to:[proteinNLet0 getLength]-1];
}

+ (DNANLet*)retranslate:(ProteinNLet*)proteinNLet0 from:(long)i0 to:(long)j0
{
  DNANLet *retranslNLet;
  long from , to;
  char *retranslSequence;
  long i;
  if(![proteinNLet0 isKindOf:[ProteinNLet class]])
    {
      [ErrorHandler error:ERRMETHODWRONGOBJ];
      return nil;
    }
  if(i0<0)
    from=0;
  else
    from=i0;
  if(j0>=[proteinNLet0 getLength])
    to=[proteinNLet0 getLength]-1;
  else
    to=j0;
  retranslSequence=(char*)malloc((3*(to-from+1)+1)*sizeof(char));
  if(!retranslSequence)
    {
      [ErrorHandler error:ERRMEMNOTENOUGH];      
      return nil;
    }
  strcpy(retranslSequence,"");
  i=from;
  while(i<=to)
    strcat(retranslSequence,[self retranslateAminoAcid:[proteinNLet0 getSequenceAt:i++]]);
  strcat(retranslSequence,"");
  retranslNLet=[DNANLet new:retranslSequence];
  free(retranslSequence);
  return retranslNLet;
}

- init:(char*)seq0
{
  BOOL validSeq;
  long i;
  self=[super init];
  if(seq0)
    {
      validSeq=YES;
      for(i=0;i<strlen(seq0);i++)
	if(validSeq && ![[self class] isInProteinAlphabet:seq0[i]])
	  validSeq=NO;
      if(validSeq)
	{
	  length=strlen(seq0);
	  sequence=(char*)realloc(sequence,(length+1)*sizeof(char));
	  if(length && !sequence)
	    {
	      [ErrorHandler error:ERRMEMNOTENOUGH];
	      return self;
	    }
	  for(i=0;i<strlen(seq0);i++)
	    sequence[i]=toupper(seq0[i]);
	  sequence[i]='\0';
	}
      else
	{
	  length=DEFNLETLEN;
	  sequence=realloc(sequence,(length+1)*sizeof(char));
	  if(length && !sequence)
	    {
	      [ErrorHandler error:ERRMEMNOTENOUGH];
	      return self;
	    }
	  strcpy(sequence,DEFNLETSEQUENCE);
	}
      for(i=0;i<SIZEASCII;i++)
	composition[i]=0;
      for(i=0;i<length;i++)
	composition[sequence[i]]++;
    }
  else
    {
      length=DEFNLETLEN;
      for(i=0;i<SIZEASCII;i++)
	composition[i]=0;
      for(i=0;i<length;i++)
	composition[sequence[i]]++;
    }
  return self;
}

- setSequence: (char*) seq0
{
  BOOL validSeq;
  long i;
  if(seq0)
    {
      validSeq=YES;
      for(i=0;i<strlen(seq0);i++)
	if(validSeq && ![[self class] isInProteinAlphabet:seq0[i]])
	  validSeq=NO;
      if(validSeq)
	{
	  length=strlen(seq0);
	  sequence=realloc(sequence,(length+1)*sizeof(char));
	  if(length && !sequence)
	    {
	      [ErrorHandler error:ERRMEMNOTENOUGH];
	      return self;
	    }
	  for(i=0;i<strlen(seq0);i++)
	    sequence[i]=toupper(seq0[i]);
	  sequence[i]='\0';
	}
      for(i=0;i<SIZEASCII;i++)
	composition[i]=0;
      for(i=0;i<length;i++)
	composition[sequence[i]]++;
    }
  return self;
}

@end
