/* Name: textexec.h                                                          **
**                                                                           **
**                                                                           **
** Funktion: Deklaration der Klasse TextExecute, die die verfuegbaren Unter- **
**           programme im Textmodus ausfuehren laesst                        **
**                                                                           **
** Aufruf: -- (Einbindung in ObjC-Dateien ueber #import bzw. #include)       **
**                                                                           **
** Probleme:                                                                 **
**                                                                           **
** Erstellt: 21.01.1996                                                      **
** Zuletzt bearbeitet: 02.07.1998                                            */

/* ------------------------------------------------------------------------- */


#ifndef TEXTEXEC_H
#define TEXTEXEC_H

/* Include-Dateien                                                           */

#include <objc/Object.h>

#include <stdio.h>

/* Konstanten- und Typdefinitionen                                           */

#define DEFSHORTHANDLEN 3

#define SHORTNOACTION ""
#define SHORTLISTLP "lis"
#define SHORTINPUTLP "ilp"
#define SHORTEDITSOURCE "esc"
#define SHORTEDITLOCALID "eid"
#define SHORTEDITDESCRIPTION "ede"
#define SHORTGENERATERANDOM "rlp"
#define SHORTLISTCOMPOSITION "lcp"
#define SHORTANTISENSE "das"
#define SHORTTRANSLATE "dtr"
#define SHORTDNAREVERSE "drv"
#define SHORTTRIPLETREVERSE "drt"
#define SHORTMININFORMATION "dmi"
#define SHORTDENSITY "dds"
#define SHORTRETRANSLATE "prt"
#define SHORTALIGNTWONLETS "atn"
#define SHORTALIGNQUERYWBULK "aqb"
#define SHORTOUTPUTALIGN "oal"
#define SHORTOUTPUTLP "olp"
#define SHORTCLOSELP "clp"
#define SHORTDBCONNECT "dbc"
#define SHORTDBDISCONNECT "dbd"
#define SHORTUNIXSH "unx"
#define SHORTEXIT "exi"

#define DEFDBCONNECTION "connect"

#define LPNOTYPE 0
#define LPDNATYPE 1
#define LPPROTEINTYPE 2

#define ALIGNALGNEEDLEWUNSCH 1
#define ALIGNALGNEIGHBOURNWUNSCH 2

#define ALMATRIXIDENTITY 1
#define ALMATRIXDNASTANDARD 11
#define ALMATRIXPAM250 21

#define FILEAUTODETECT 0
#define FILEPLAINDNA 101
#define FILEEMBL 102
#define FILEPLAINPROTEIN 201
#define FILESWISSPROT 202

/* Deklarierte Klassen                                                       */

@class TextExecute;

/* Externe Klassen                                                           */

@class LinkedList;
@class LinearPolymer;
@class DNA;
@class Protein;

/* Klasse: TextExecute                                                       */

@interface TextExecute : Object
{
  BOOL useDB;
  char *dbServer;
  FILE *logFile;
  LinkedList *linPolymerList;
  int curQueryLPolymer;
  LinkedList *curSearchList;
  LinkedList *curAlignmentList;
}

+ new:(char*)server0 logFile:(FILE*)logFile0;
+ alloc;
- init:(char*)server0 logFile:(FILE*)logFile0;
- (BOOL)getDBUsage;
- (char*)getDBServer;
- (int)executeByShorthand:(char*)short0;
- (int)chooseLinearPolymer:(int)type0;
- (int)chooseLPolymerFromList;
- (int)listOpenLPolymers;
- (char*)buildSelectStatement;
- (int)inputLinearPolymer:(int)type0;
- (int)generateRandomLPolymer:(int)type0;
- (int)inputLPolymerFromDB:(int)type0;
- (int)inputLPolymerFromFile:(int)type0;
- (int)inputLPolymerManually:(int)type0;
- (DNA*)inputDNAFile:(FILE*)file0 format:(int)format0;
- (Protein*)inputProteinFile:(FILE*)file0 format:(int)format0;
- (DNA*)inputPlainDNAFile:(FILE*)file0;
- (DNA*)inputEMBLFile:(FILE*)file0;
- (Protein*)inputPlainProteinFile:(FILE*)file0;
- (Protein*)inputSWISSPROTFile:(FILE*)file0;
- (int)editSource;
- (int)editLocalID;
- (int)editDescription;
- (int)listComposition;
- (int)dnaAntisenseStrand;
- (int)dnaTranslate;
- (int)dnaReverse;
- (int)dnaTripletReverse;
- (int)dnaMinimumInformation;
- (int)dnaInformationDensity;
- (int)proteinRetranslate;
- (int)chooseAlignmentAlgorithm;
- (int)chooseAlignmentMatrix:(int)type0;
- (int)alignTwoNLets;
- (int)alignQueryNLetWBulk;
- (int)alignQueryNLetWBulkFromDB:(LinearPolymer*)lPolymer0 from:(long)i0 to:(long)j0;
- (int)alignQueryNLetWBulkFromFile:(LinearPolymer*)lPolymer0 from:(long)i0 to:(long)j0;
- (LinearPolymer*)fileFetch:(FILE*)file0 type:(int)type0 format:(int)format0;
- (int)outputAlignments;
- (int)outputLinearPolymer;
- (int)outputLinearPolymerToFile:(LinearPolymer*)lPolymer0;
- (int)output:(DNA*)dna0 toPlainDNAFile:(FILE*)file0;
- (int)output:(DNA*)dna0 toEMBLFile:(FILE*)file0;
- (int)output:(Protein*)protein0 toPlainProteinFile:(FILE*)file0;
- (int)output:(Protein*)protein0 toSWISSPROTFile:(FILE*)file0;
- (int)closeLinearPolymer;
- (int)dbConnect;
- (int)dbDisconnect;
- (int)openUNIXShellCommand;
- free;

@end

#endif
