/* Name: textmode.c                                                          **
**                                                                           **
**                                                                           **
** Funktion: Bereitstellen von Funktionen zur Ein- und Ausgabe im Textmodus  **
**                                                                           **
** Aufruf: -- (Linken der .o Datei zu Programmen)                            **
**                                                                           **
** Probleme:                                                                 **
**                                                                           **
** Erstellt: 03.01.1996                                                      **
** Zuletzt bearbeitet: 17.11.1997                                            */

/* ------------------------------------------------------------------------- */


/* Include-Dateien                                                           */

#include "constant.h"

#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include <stdlib.h>

/* Interface                                                                 */

/* outputTitle: gibt den Titel title0 eines Anwendungsprogrammes aus         */
int outputTitle(char *title0);

/* outputExitMessage: gibt die Nachricht exitMsg0 aus                        */
int outputExitMessage(char *exitMsg0);

/* readLoginData: liest Zugang und Passwort in login und passwd              */
int readLoginData(char *login, char* passwd);

/* outputMenuTitle: gibt den Titel title0 eines Menues aus                   */
int outputMenuTitle(char *title0);

/* outputMenuDescription: gibt die Beschreibung desc0 zu einem Menue aus     */
int outputMenuDescription(char *desc0);

/* outputMenuItem: gibt einen Menuepunkt aus, wobei item0 die Nummer des     **
**                 Punktes, letter0 das Buchstabenkuerzel, shortcut das      **
**                 Zeichenkettenkuerzel und menuPoint0 die Beschreibung ist  */
int outputMenuItem(int item0,char letter0,char *shortcut0,char *menuPoint0);

/* readMenuChoice: liest die Menueauswahl des Nutzers in choice              */
int readMenuChoice(char *choice);

/* readChar: liest ein Zeichen aus der Datei / dem Strom file0               */
char readChar(FILE *file0);

/* readCharWCommand: liest ein Zeichen aus file0 , wobei das Kommando        **
**                   command0 nach stdout gegeben wird                       */
char readCharWCommand(FILE *file0,char *command0);

/* readCharWCommandDefault: liest ein Zeichen aus file0, wobei das Kommando  **
**                          command0 nach stdout gegeben und der             **
**                          Standardwert default0 verwendet wird             */
char readCharWCommandDefault(FILE *file0,char *command0,char default0);

/* readInt: liest eine Ganzzahl aus der Datei / dem Strom file0              */
int readInt(FILE *file0);

/* readIntWCommand: liest eine Ganzzahl aus file0 , wobei das Kommando       **
**                  command0 nach stdout gegeben wird                        */
int readIntWCommand(FILE *file0,char *command0);

/* readIntWCommandDefault: liest eine Ganzzahl aus file0, wobei das Kommando **
**                         command0 nach stdout gegeben und der Standardwer  **
**                         default0 verwendet wird                           */
int readIntWCommandDefault(FILE *file0,char *command0,int default0);

/* readLong: liest eine lange Ganzzahl aus der Datei / dem Strom file0       */
long readLong(FILE *file0);

/* readLongWCommand: liest eine lange Ganzzahl aus file0 , wobei das         **
**                  Kommando command0 nach stdout gegeben wird               */
long readLongWCommand(FILE *file0,char *command0);

/* readLongWCommandDefault: liest eine lange Ganzzahl aus file0, wobei das   **
**                          Kommando command0 nach stdout gegeben und der    **
**                          Standardwert default0 verwendet wird             */
long readLongWCommandDefault(FILE *file0,char *command0,long default0);

/* readFloat: liest eine Fliesskommazahl aus der Datei / dem Strom file0     */
float readFloat(FILE *file0);

/* readFloatWCommand: liest eine Fliesskommazahl aus file0 , wobei das       **
**                    Kommando command0 nach stdout gegeben wird             */
float readFloatWCommand(FILE *file0,char *command0);

/* readFloatWCommandDefault: liest eine Fliesskommazahl aus file0, wobei das **
**                           Kommando command0 nach stdout gegeben und der   **
**                           Standardwert default0 verwendet wird            */
float readFloatWCommandDefault(FILE *file0,char *command0,float default0);

/* readFloat: liest eine Fliesskommazahl mit doppelter Genauigkeit aus file0 */
double readDouble(FILE *file0);

/* readFloatWCommand: liest eine genaue Fliesskommazahl aus file0 , wobei    **
**                    das Kommando command0 nach stdout gegeben wird         */
double readDoubleWCommand(FILE *file0,char *command0);

/* readFloatWCommandDefault: liest eine genaue Fliesskommazahl aus file0,    **
**                           wobei das Kommando command0 nach stdout gegeben **
**                           und der Standardwert default0 verwendet wird    */
double readDoubleWCommandDefault(FILE *file0,char *command0,double default0);

/* readFloat: liest eine Zeichenkette aus file0 in die Variable string       */
int readString(char *string,FILE *file0);

/* readStringWCommand: liest eine Zeichenkette aus file0 in string, wobei    **
**                     das Kommando command0 nach stdout gegeben wird        */
int readStringWCommand(char *string,FILE *file0,char *command0);

/* readStringWCommandDefault: liest eine Zeichenkette aus file0 in string,   **
**                            wobei das Kommando command0 nach stdout        **
**                            gegeben und der Standardwert default0          **
**                            verwendet wird                                 */
int readStringWCommandDefault(char *string,FILE *file0,char *command0,char *default0);

/* readStringWCommandHead: liest eine Zeichenkette aus file0 in string,      **
**                         wobei das Kommando command0 nach stdout gegeben   **
**                         wird und die Zeichenkette head0 als Anfang des    **
**                         Rueckgabewertes feststeht                         */
int readStringWCommandHead(char *string,FILE *file0,char *command0,char *head0);

/* errorMessage: gibt die Fehlermeldung errorMsg0 nach stderr aus            */
int errorMessage(char *errorMsg0);

/* Implementation                                                            */

int outputTitle(char *title0)
{
  printf("\n");
  printf("%s\n",title0);
  printf("\n\n");
  return RETURNEXIT;
}

int outputExitMessage(char *exitMsg0)
{
  printf("\n");
  printf("%s\n",exitMsg0);
  printf("\n");
  return RETURNEXIT;
}

int readLoginData(char *login, char* passwd)
{
  char *usrInput;
  char check;
  usrInput=(char*)malloc(DEFSTRINGLEN+1);
  printf("\n");
  readStringWCommand(login,stdin,"Please enter the login name");
  /* "Unsichtbare" Eingabe des Passwortes */
  strcpy(passwd,(char*)getpass("Please enter the password: "));
  /* "Sichtbare" Eingabe des Passwortes */
  /* readStringWCommand(passwd,stdin,"Please enter the password"); */
  printf("\n\n");
  free(usrInput);
  return RETURNEXIT;
}

int outputMenuTitle(char *title0)
{
  char *title;
  int titleLength;
  int i;
  title=(char*)malloc(DEFSTRINGLEN+1);
  titleLength=strlen(title0);
  for(i=0;i<titleLength;i++)
    *(title+i)=toupper(*(title0++));
  *(title+titleLength)='\0';
  printf("\n");
  printf("%s\n",title);
  printf("\n");
  free(title);
  return RETURNEXIT;
}

int outputMenuDescription(char *desc0)
{
  char *desc, *line;
  int i;
  desc=(char*)malloc(DEFSTRINGLEN+1);
  line=(char*)malloc(TXTMODECHARPERLINE+1);
  strcpy(desc,desc0);
  while(strlen(desc)>0)
    if(strlen(desc)<TXTMODECHARPERLINE)
      /* Beschreibung / Rest der Beschreibung passt auf eine Zeile */
      {
	printf("%s\n",desc);
	printf("\n");
	strcpy(desc,"");
	}
    else
      {
	for(i=0;i<=TXTMODECHARPERLINE;line[i++]='\0');
	for(i=TXTMODECHARPERLINE-1;*(desc+i)!=' ';i--);
	strncpy(line,desc,i);
	strcat(line,"\n");
	printf("%s",line);
	strcpy(desc,desc+i+1);
      }
  free(desc);
  free(line);
  return RETURNEXIT;
}

int outputMenuItem(int item0,char letter0,char *shortcut0,char *menuPoint0)
{
  printf("%2d   %c   [%-3.3s]   %.60s\n",item0,letter0,shortcut0,menuPoint0);
  return RETURNEXIT;
}

int readMenuChoice(char *menuChoice)
{
  printf("\n");
  readStringWCommand(menuChoice,stdin,"Please enter your choice");
  printf("\n");
  return RETURNEXIT;
}

char readChar(FILE *file0)
{
  char returnChar;
  char *usrInput;
  char check;
  usrInput=(char*)malloc(DEFSTRINGLEN+1);
  do
    {
      fgets(usrInput,DEFSTRINGLEN,file0);
      check=sscanf(usrInput,"%c",&returnChar);
    }
  while(check!=1);
  /* ...bis Eingabe erfolgt ist */
  fflush(stdin);
  free(usrInput);
  return returnChar;
}

char readCharWCommand(FILE *file0,char *command0)
{
  char returnChar;
  char *usrInput;
  char check;
  usrInput=(char*)malloc(DEFSTRINGLEN+1);
  do
    {
      /* Ausgabe des Kommandos */
      printf("%s: ",command0);
      fgets(usrInput,DEFSTRINGLEN,file0);
      check=sscanf(usrInput,"%c",&returnChar);
    }
  while(check!=1);
  fflush(stdin);
  free(usrInput);
  return returnChar;
}

char readCharWCommandDefault(FILE *file0,char *command0,char default0)
{
  char returnChar;
  char *usrInput;
  usrInput=(char*)malloc(DEFSTRINGLEN+1);
  /* Ausgabe des Kommandos und des Standardwertes */
  printf("%s (* %c *): ",command0,default0);
  fgets(usrInput,DEFSTRINGLEN,file0);
  sscanf(usrInput,"%c",&returnChar);
  if(returnChar=='\n')
    /* keine Eingabe erfolgt, also Standardwert verwenden */
    returnChar=default0;
  fflush(stdin);
  free(usrInput);
  return returnChar;
}

int readInt(FILE *file0)
{
  int returnInt;
  char *usrInput;
  char check;
  usrInput=(char*)malloc(DEFSTRINGLEN+1);
  do
    {
      fgets(usrInput,DEFSTRINGLEN,file0);
      check=sscanf(usrInput,"%d",&returnInt);
    }
  while(check!=1);
  fflush(stdin);
  free(usrInput);
  return returnInt;
}

int readIntWCommand(FILE *file0,char *command0)
{
  int returnInt;
  char *usrInput;
  char check;
  usrInput=(char*)malloc(DEFSTRINGLEN+1);
  do
    {
      printf("%s: ",command0);
      fgets(usrInput,DEFSTRINGLEN,file0);
      check=sscanf(usrInput,"%d",&returnInt);
    }
  while(check!=1);
  fflush(stdin);
  free(usrInput);
  return returnInt;
}

int readIntWCommandDefault(FILE *file0,char *command0,int default0)
{
  int returnInt;
  char *usrInput;
  char check;
  usrInput=(char*)malloc(DEFSTRINGLEN+1);
  printf("%s (* %d *): ",command0,default0);
  fgets(usrInput,DEFSTRINGLEN,file0);
  check=sscanf(usrInput,"%d",&returnInt);
  if(check!=1)
    /* keine Eingabe erfolgt, also Standardwert verwenden */
    returnInt=default0;
  fflush(stdin);
  free(usrInput);
  return returnInt;
}

long readLong(FILE *file0)
{
  long returnLong;
  char *usrInput;
  char check;
  usrInput=(char*)malloc(DEFSTRINGLEN+1);
  do
    {
      fgets(usrInput,DEFSTRINGLEN,file0);
      check=sscanf(usrInput,"%ld",&returnLong);
    }
  while(check!=1);
  fflush(stdin);
  free(usrInput);
  return returnLong;
}

long readLongWCommand(FILE *file0,char *command0)
{
  long returnLong;
  char *usrInput;
  char check;
  usrInput=(char*)malloc(DEFSTRINGLEN+1);
  do
    {
      printf("%s: ",command0);
      fgets(usrInput,DEFSTRINGLEN,file0);
      check=sscanf(usrInput,"%ld",&returnLong);
    }
  while(check!=1);
  fflush(stdin);
  free(usrInput);
  return returnLong;
}

long readLongWCommandDefault(FILE *file0,char *command0,long default0)
{
  long returnLong;
  char *usrInput;
  char check;
  usrInput=(char*)malloc(DEFSTRINGLEN+1);
  printf("%s (* %ld *): ",command0,default0);
  fgets(usrInput,DEFSTRINGLEN,file0);
  check=sscanf(usrInput,"%ld",&returnLong);
  if(check!=1)
    returnLong=default0;
  fflush(stdin);
  free(usrInput);
  return returnLong;
}

float readFloat(FILE *file0)
{
  float returnFloat;
  char *usrInput;
  char check;
  usrInput=(char*)malloc(DEFSTRINGLEN+1);
  do
    {
      fgets(usrInput,DEFSTRINGLEN,file0);
      check=sscanf(usrInput,"%f",&returnFloat);
    }
  while(check!=1);
  fflush(stdin);
  free(usrInput);
  return returnFloat;
}

float readFloatWCommand(FILE *file0,char *command0)
{
  float returnFloat;
  char *usrInput;
  char check;
  usrInput=(char*)malloc(DEFSTRINGLEN+1);
  do
    {
      printf("%s: ",command0);
      fgets(usrInput,DEFSTRINGLEN,file0);
      check=sscanf(usrInput,"%f",&returnFloat);
    }
  while(check!=1);
  fflush(stdin);
  free(usrInput);
  return returnFloat;
}

float readFloatWCommandDefault(FILE *file0,char *command0,float default0)
{
  float returnFloat;
  char *usrInput;
  char check;
  usrInput=(char*)malloc(DEFSTRINGLEN+1);
  printf("%s (* %f *): ",command0,default0);
  fgets(usrInput,DEFSTRINGLEN,file0);
  check=sscanf(usrInput,"%f",&returnFloat);
  if(check!=1)
    returnFloat=default0;
  fflush(stdin);
  free(usrInput);
  return returnFloat;
}

double readDouble(FILE *file0)
{
  double returnDouble;
  char *usrInput;
  char check;
  usrInput=(char*)malloc(DEFSTRINGLEN+1);
  do
    {
      fgets(usrInput,DEFSTRINGLEN,file0);
      check=sscanf(usrInput,"%f",&returnDouble);
    }
  while(check!=1);
  fflush(stdin);
  free(usrInput);
  return returnDouble;
}

double readDoubleWCommand(FILE *file0,char *command0)
{
  double returnDouble;
  char *usrInput;
  char check;
  usrInput=(char*)malloc(DEFSTRINGLEN+1);
  do
    {
      printf("%s: ",command0);
      fgets(usrInput,DEFSTRINGLEN,file0);
      check=sscanf(usrInput,"%f",&returnDouble);
    }
  while(check!=1);
  fflush(stdin);
  free(usrInput);
  return returnDouble;
}

double readDoubleWCommandDefault(FILE *file0,char *command0,double default0)
{
  double returnDouble;
  char *usrInput;
  char check;
  usrInput=(char*)malloc(DEFSTRINGLEN+1);
  printf("%s (* %f *): ",command0,default0);
  fgets(usrInput,DEFSTRINGLEN,file0);
  check=sscanf(usrInput,"%f",&returnDouble);
  if(check!=1)
    returnDouble=default0;
  fflush(stdin);
  free(usrInput);
  return returnDouble;
}

int readString(char *string,FILE *file0)
{
  char *usrInput;
  string=(char*)malloc(DEFSTRINGLEN+1);
  usrInput=(char*)malloc(DEFSTRINGLEN+1);
  do
    fgets(usrInput,DEFSTRINGLEN,file0);
  while(usrInput[0]=='\n');
  /* ...bis Eingabe erfolgt ist */
  strcpy(string,usrInput);
  if(string[strlen(string)-1]=='\n')
    /* Abschneiden des Zeilenumbruchs */
    string[strlen(string)-1]='\0';
  else
    string[strlen(string)]='\0';
  fflush(stdin);
  free(usrInput);
  return RETURNEXIT;
}

int readStringWCommand(char *string,FILE *file0,char *command0)
{
  char *usrInput;
  usrInput=(char*)malloc(DEFSTRINGLEN+1);
  do
    {
      printf("%s: ",command0);
      fgets(usrInput,DEFSTRINGLEN,file0);
    }
  while(usrInput[0]=='\n');
  strcpy(string,usrInput);
  if(string[strlen(string)-1]=='\n')
    string[strlen(string)-1]='\0';
  else
    string[strlen(string)]='\0';
  fflush(stdin);
  free(usrInput);
  return RETURNEXIT;
}

int readStringWCommandDefault(char *string,FILE *file0,char *command0,char *default0)
{
  char *usrInput;
  usrInput=(char*)malloc(DEFSTRINGLEN+1);
  printf("%s (* %s *): ",command0,default0);
  fgets(usrInput,DEFSTRINGLEN,file0);
  if(usrInput[0]=='\n')
    /* keine Eingabe erfolgt, also Standardwert verwenden */
    strcpy(string,default0);
  else
    strcpy(string,usrInput);
  if(string[strlen(string)-1]=='\n')
    string[strlen(string)-1]='\0';
  else
    string[strlen(string)]='\0';
  fflush(stdin);
  free(usrInput);
  return RETURNEXIT;
}

int readStringWCommandHead(char *string,FILE *file0,char *command0,char *head0)
{
  char *usrInput;
  usrInput=(char*)malloc(DEFSTRINGLEN+1);
  printf("%s: %s",command0,head0);
  fgets(usrInput,DEFSTRINGLEN,file0);
  /* Verwenden des festen Kopfes */
  strcpy(string,head0);
  if(usrInput[0]!='\n')
    strcat(string,usrInput);
  if(string[strlen(string)-1]=='\n')
    string[strlen(string)-1]='\0';
  else
    string[strlen(string)]='\0';
  fflush(stdin);
  free(usrInput);
  return RETURNEXIT;
}

int errorMessage(char *errorMsg0)
{
  fprintf(stderr,"Error occured: %s\n",errorMsg0);
  return RETURNEXIT;
}
